@@ -28,13 +28,14 @@ import java.util.Map;
 import java.util.Set;
 import org.apache.directory.api.ldap.model.entry.Entry;
 import org.apache.directory.api.ldap.model.exception.LdapInvalidAttributeValueException;
+import org.apache.directory.api.ldap.model.filter.AndNode;
 import org.apache.directory.api.ldap.model.filter.EqualityNode;
 import org.apache.directory.api.ldap.model.filter.ExprNode;
 import org.apache.directory.api.ldap.model.filter.NotNode;
-import org.apache.directory.api.ldap.model.filter.PresenceNode;
 import org.apache.directory.api.ldap.model.name.Dn;
 import org.apache.directory.ldap.client.api.LdapNetworkConnection;
 import org.apache.guacamole.auth.ldap.conf.ConfigurationService;
+import org.apache.guacamole.auth.ldap.conf.MemberAttributeType;
 import org.apache.guacamole.GuacamoleException;
 import org.apache.guacamole.auth.ldap.ObjectQueryService;
 import org.apache.guacamole.net.auth.UserGroup;
@@ -80,16 +81,22 @@ public class UserGroupService {
      */
     private ExprNode getGroupSearchFilter() throws GuacamoleException {
 
+        // Use filter defined by "ldap-group-search-filter" as basis for all
+        // retrieval of user groups
+        ExprNode groupFilter = confService.getGroupSearchFilter();
+
         // Explicitly exclude guacConfigGroup object class only if it should
         // be assumed to be defined (query may fail due to no such object
         // class existing otherwise)
-        if (confService.getConfigurationBaseDN() != null)
-            return new NotNode(new EqualityNode("objectClass","guacConfigGroup"));
-
-        // Read any object as a group if LDAP is not being used for connection
-        // storage (guacConfigGroup)
-        return new PresenceNode("objectClass");
+        if (confService.getConfigurationBaseDN() != null) {
+            groupFilter = new AndNode(
+                groupFilter,
+                new NotNode(new EqualityNode<String>("objectClass", "guacConfigGroup"))
+            );
+        }
 
+        return groupFilter;
+        
     }
 
     /**
@@ -116,6 +123,11 @@ public class UserGroupService {
         if (groupBaseDN == null)
             return Collections.emptyMap();
 
+        // Gather all attributes relevant for a group
+        String memberAttribute = confService.getMemberAttribute();
+        Collection<String> groupAttributes = new HashSet<>(confService.getGroupNameAttributes());
+        groupAttributes.add(memberAttribute);
+
         // Retrieve all visible user groups which are not guacConfigGroups
         Collection<String> attributes = confService.getGroupNameAttributes();
         List<Entry> results = queryService.search(
@@ -123,7 +135,8 @@ public class UserGroupService {
             groupBaseDN,
             getGroupSearchFilter(),
             attributes,
-            null
+            null,
+            groupAttributes
         );
 
         // Convert retrieved user groups to map of identifier to Guacamole
@@ -176,14 +189,52 @@ public class UserGroupService {
         if (groupBaseDN == null)
             return Collections.emptyList();
 
+        // memberAttribute specified in properties could contain DN or username 
+        MemberAttributeType memberAttributeType = confService.getMemberAttributeType();
+        String userIDorDN = userDN.toString();
+        Collection<String> userAttributes = confService.getUsernameAttributes();
+        if (memberAttributeType == MemberAttributeType.UID) {
+            // Retrieve user objects with userDN
+            List<Entry> userEntries = queryService.search(
+                ldapConnection,
+                userDN,
+                confService.getUserSearchFilter(),
+                0,
+                userAttributes);
+            // ... there can surely only be one
+            if (userEntries.size() != 1)
+                logger.warn("user DN \"{}\" does not return unique value "
+                        + "and will be ignored", userDN.toString());
+            else {
+                // determine unique identifier for user
+                Entry userEntry = userEntries.get(0);
+                try {
+                    userIDorDN = queryService.getIdentifier(userEntry,
+                                         userAttributes);
+                }
+                catch (LdapInvalidAttributeValueException e) {
+                    logger.error("User group missing identifier: {}",
+                            e.getMessage());
+                    logger.debug("LDAP exception while getting "
+                            + "group identifier.", e);
+                }
+            }
+        }
+
+        // Gather all attributes relevant for a group
+        String memberAttribute = confService.getMemberAttribute();
+        Collection<String> groupAttributes = new HashSet<>(confService.getGroupNameAttributes());
+        groupAttributes.add(memberAttribute);
+
         // Get all groups the user is a member of starting at the groupBaseDN,
         // excluding guacConfigGroups
         return queryService.search(
             ldapConnection,
             groupBaseDN,
             getGroupSearchFilter(),
-            Collections.singleton(confService.getMemberAttribute()),
-            userDN.toString()
+            Collections.singleton(memberAttribute),
+            userIDorDN,
+            groupAttributes
         );
 
     }
