@@ -20,12 +20,15 @@
 package org.apache.guacamole.protocol;
 
 import java.util.List;
+import org.apache.guacamole.GuacamoleConnectionClosedException;
 import org.apache.guacamole.GuacamoleException;
 import org.apache.guacamole.GuacamoleServerException;
 import org.apache.guacamole.io.GuacamoleReader;
 import org.apache.guacamole.io.GuacamoleWriter;
 import org.apache.guacamole.net.DelegatingGuacamoleSocket;
 import org.apache.guacamole.net.GuacamoleSocket;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * A GuacamoleSocket which pre-configures the connection based on a given
@@ -39,6 +42,12 @@ import org.apache.guacamole.net.GuacamoleSocket;
  */
 public class ConfiguredGuacamoleSocket extends DelegatingGuacamoleSocket {
 
+    /**
+     * Logger for this class.
+     */
+    private static final Logger logger =
+            LoggerFactory.getLogger(ConfiguredGuacamoleSocket.class);
+
     /**
      * The configuration to use when performing the Guacamole protocol
      * handshake.
@@ -59,17 +68,83 @@ public class ConfiguredGuacamoleSocket extends DelegatingGuacamoleSocket {
      */
     private GuacamoleProtocolVersion protocolVersion =
             GuacamoleProtocolVersion.VERSION_1_0_0;
-    
+
+    /**
+     * Parses the given "error" instruction, throwing a GuacamoleException that
+     * corresponds to its status code and message.
+     *
+     * @param instruction
+     *     The "error" instruction to parse.
+     *
+     * @throws GuacamoleException
+     *     A GuacamoleException that corresponds to the status code and message
+     *     present within the given "error" instruction.
+     */
+    private static void handleReceivedError(GuacamoleInstruction instruction)
+            throws GuacamoleException {
+
+        // Provide reasonable default error message for invalid "error"
+        // instructions that fail to provide one
+        String message = "Internal error within guacd / protocol handling.";
+
+        // Consider all error instructions without a corresponding status code
+        // to be server errors
+        GuacamoleStatus status = GuacamoleStatus.SERVER_ERROR;
+
+        // Parse human-readable message from "error" instruction, warning if no
+        // message was given
+        List<String> args = instruction.getArgs();
+        if (args.size() >= 1)
+            message = args.get(0);
+        else
+            logger.debug("Received \"error\" instruction with no corresponding message.");
+
+        // Parse the status code from the received error instruction, warning
+        // if the status code is missing or invalid
+        if (args.size() >= 2) {
+            try {
+                
+                // Translate numeric status code into a GuacamoleStatus
+                int statusCode = Integer.parseInt(args.get(1));
+                GuacamoleStatus parsedStatus = GuacamoleStatus.fromGuacamoleStatusCode(statusCode);
+                if (parsedStatus != null)
+                    status = parsedStatus;
+                else
+                    logger.debug("Received \"error\" instruction with unknown/invalid status code: {}", statusCode);
+
+            }
+            catch (NumberFormatException e) {
+                logger.debug("Received \"error\" instruction with non-numeric status code.", e);
+            }
+        }
+        else
+            logger.debug("Received \"error\" instruction without status code.");
+
+        // Convert parsed status code and message to a GuacamoleException
+        throw status.toException(message);
+
+    }
+
     /**
      * Waits for the instruction having the given opcode, returning that
      * instruction once it has been read. If the instruction is never read,
      * an exception is thrown.
-     * 
-     * @param reader The reader to read instructions from.
-     * @param opcode The opcode of the instruction we are expecting.
-     * @return The instruction having the given opcode.
-     * @throws GuacamoleException If an error occurs while reading, or if
-     *                            the expected instruction is not read.
+     *
+     * Respects server control instructions that are allowed during the handshake
+     * phase, namely {@code error} and {@code disconnect}.
+     *
+     * @param reader
+     *     The reader to read instructions from.
+     *
+     * @param opcode
+     *     The opcode of the instruction we are expecting.
+     *
+     * @return
+     *     The instruction having the given opcode.
+     *
+     * @throws GuacamoleException
+     *     If an error occurs while reading, or if the expected instruction is
+     *     not read.
      */
     private GuacamoleInstruction expect(GuacamoleReader reader, String opcode)
         throws GuacamoleException {
@@ -79,6 +154,14 @@ public class ConfiguredGuacamoleSocket extends DelegatingGuacamoleSocket {
         if (instruction == null)
             throw new GuacamoleServerException("End of stream while waiting for \"" + opcode + "\".");
 
+        // Report connection closure if server explicitly disconnects
+        if ("disconnect".equals(instruction.getOpcode()))
+            throw new GuacamoleConnectionClosedException("Server disconnected while waiting for \"" + opcode + "\".");
+
+        // Pass through any received errors as GuacamoleExceptions
+        if ("error".equals(instruction.getOpcode()))
+            handleReceivedError(instruction);
+
         // Ensure instruction has expected opcode
         if (!instruction.getOpcode().equals(opcode))
             throw new GuacamoleServerException("Expected \"" + opcode + "\" instruction but instead received \"" + instruction.getOpcode() + "\".");
